@@ -2,11 +2,13 @@
 Client to submit jobs to and retrieve results from a remote ngscheck service.
 """
 import argparse
-import getpass
+import getpass as getpassmodule
 import logging
 import sys
 
-from . import api
+from os.path import basename
+
+from . import api, util
 
 
 LOGGER = logging.getLogger(__name__)
@@ -16,31 +18,38 @@ def main():
     """
     Top-level control flow.
     """
-    args = get_args(sys.argv[1:])
+    parser = get_parser()
+    args = parser.parse_args()
     logging.basicConfig(level=args.loglevel)
     exit_code = 0
     try:
-        if args.password_file:
-            password = args.password_file.read().strip()
-            args.password_file.close()
-        else:
-            password = args.password or getpass.getpass("Password: ")
-        token = api.get_access_token(args.email, password)
-        userid = api.get_userid(token)
-        command = args.subcommand
-        if command == 'submit':
+        action = args.action
+        # auth stuff common to API actions
+        if hasattr(args, 'email'):
+            password = getpass(args.password_file)
+            token = api.get_access_token(args.email, password)
+            userid = api.get_userid(token)
+
+        if action == 'validate':
+            invalid = [f.name for f in args.files if not util.validate(f)]
+            if invalid:
+                LOGGER.error("invalid files: %s", invalid)
+                exit_code = 2
+            else:
+                LOGGER.info("all files are valid")
+        elif action == 'submit':
             api.submit(args.files, args.manifest_file, userid, token)
-        elif command == 'get_status':
+        elif action == 'get_status':
             api.get_status(args.manifest_file, userid, token)
-        elif command == 'get_results':
+        elif action == 'get_results':
             # get_status first to get latest status
             api.get_status(args.manifest_file, userid, token)
             api.get_results(args.manifest_file, args.include_logs,
                             userid, token)
-        elif command == 'get_userid':
+        elif action == 'get_userid':
             print(userid)
         else:
-            raise NotImplementedError(command)
+            raise NotImplementedError(action)
 
     except Exception as ex:
         LOGGER.exception(ex)
@@ -49,33 +58,13 @@ def main():
     sys.exit(exit_code)
 
 
-def get_args(args):
+def get_parser():
     """
-    Parse arguments from the command line.
-
-    Args:
-        args: options & arguments from the command line, e.g. sys.argv[1:]
-
-    Returns:
-        argparse.Namespace
+    Return a configured ArgumentParser for command-line use.
     """
+    my_name = basename(sys.argv[0])
     parser = argparse.ArgumentParser(description=__doc__)
-
-    common_args = parser.add_argument_group('common arguments')
-    common_args.add_argument(
-        "--email",
-        required=True,
-        help="email used to sign up to the service")
-    password_group = common_args.add_mutually_exclusive_group()
-    password_group.add_argument(
-        "--password",
-        help="using --password-file or interactive use is safer")
-    password_group.add_argument(
-        "--password-file",
-        type=argparse.FileType('r'),
-        help="if neither --password nor --password-file is supplied a "
-        "password will be prompted for interactively")
-    common_args.add_argument(
+    parser.add_argument(
         "--verbose",
         action="store_const",
         dest="loglevel",
@@ -84,14 +73,36 @@ def get_args(args):
         help="output more logging (INFO level)")
 
     subparsers = parser.add_subparsers(
-        dest="subcommand",
+        dest="action",
+        title="actions",
         required=True,
-        help="SUBCOMMAND -h for subcommand help")
+        help=f"{my_name} ACTION -h for help on individual actions")
+
+    validate = subparsers.add_parser(
+        "validate",
+        help="validate qprofiler2 outputs")
+    validate.add_argument(
+        "files",
+        metavar="QPXML",
+        type=argparse.FileType('rb'),
+        nargs='+')
+
+    # all API actions require auth
+    api_actions = argparse.ArgumentParser(add_help=False)
+    api_actions.add_argument(
+        "--email",
+        required=True,
+        help="email used to sign up to the service")
+    api_actions.add_argument(
+        "--password-file",
+        type=argparse.FileType('r'),
+        help="if --password-file is not supplied a password will be prompted "
+        "for interactively")
 
     submit_p = subparsers.add_parser(
         "submit",
-        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
-        help="submit qprofiler outputs to be analyzed and generate job "
+        parents=[api_actions],
+        help="submit qprofiler2 outputs to be analyzed and generate job "
         "manifest file")
     submit_p.add_argument(
         "files",
@@ -102,11 +113,11 @@ def get_args(args):
         "--manifest-file",
         # exclusive new write (fail if exists)
         type=argparse.FileType('x'),
-        default="manifest.json",
         help="name of the job manifest file that will be created")
 
     get_status_p = subparsers.add_parser(
         "get_status",
+        parents=[api_actions],
         help="check and update status of jobs listed in a manifest file")
     get_status_p.add_argument(
         "--manifest-file",
@@ -115,6 +126,7 @@ def get_args(args):
 
     get_results_p = subparsers.add_parser(
         "get_results",
+        parents=[api_actions],
         help="get results of completed jobs listed in a manifest file")
     get_results_p.add_argument(
         "--manifest-file",
@@ -127,9 +139,25 @@ def get_args(args):
 
     subparsers.add_parser(
         "get_userid",
+        parents=[api_actions],
         help="return user identifier associated with this account (FYI only)")
 
-    return parser.parse_args(args)
+    return parser
+
+
+def getpass(passwordfile=None):
+    """
+    Returns password read from file or interactively from prompt.
+
+    Args:
+        passwordfile: an open file-like object, will be closed after reading
+    """
+    if passwordfile:
+        password = passwordfile.readline().strip()
+        passwordfile.close()
+    else:
+        password = getpassmodule.getpass('Password: ')
+    return password
 
 
 if __name__ == "__main__":
